def createPathGoesData(bucket, year, month, day, product, hour, channel=None):
    """ The modules reponsive about the has files on the new directory created with the createPathGoesData,
    and if has a same File, check if are broken"""
    """Function used create a directory, with the calendar date the user give to function"""

    from goespy import os
    from pathlib import Path

    pathReturn = ''
    '''that part will get your home directory and the Satellite bucket you're getting your dataset
                                                                                               '''
    satGoesPath = bucket.partition('noaa-')[2]

    home = str(Path.home())
    if channel == None:

        if not os.path.exists("{0}/{1}/{2}/{3}/{4}/{5}/{6}/".format(
                home, satGoesPath, year, month, day, product, hour)):

            os.makedirs(
                "{0}/{1}/{2}/{3}/{4}/{5}/{6}/".format(
                    home, satGoesPath, year, month, day, product, hour),
                exist_ok=True)

        pathReturn = "{0}/{1}/{2}/{3}/{4}/{5}/{6}/".format(
            home, satGoesPath, year, month, day, product, hour)

    else:

        if os.path.exists("{0}/{1}/{2}/{3}/{4}/{5}/{6}/{7}/".format(
                home, satGoesPath, year, month, day, product, hour, channel)):
            pass

        else:

            os.makedirs(
                "{0}/{1}/{2}/{3}/{4}/{5}/{6}/{7}/".format(
                    home, satGoesPath, year, month, day, product, hour,
                    channel),
                exist_ok=True)

        pathReturn = "{0}/{1}/{2}/{3}/{4}/{5}/{6}/{7}/".format(
            home, satGoesPath, year, month, day, product, hour, channel)

    return pathReturn


def checkFiles(path, singleFile):
    """   checkFiles(path,singleFile): That function will check if has files on the recent created path with the
                                createPathGoesData function   
                    The variable path is created with the createPathGoesData
                The singleFile is the filename from the data in the bucket 'noaa-goes16'
                     and will check the singleFile exists on the created path 
    """
    from goespy import os
    if os.path.isfile("{0}/{1}".format(path, singleFile)):
        return True
    else:
        return False


""" The modules reponsive about existence of files on the new directory created with the createPathGoesData,
    and if has a same File, check if is broken"""


def checkSize(path, singleFile, singleSize):
    """  
              That function will check if the file on the path created with the
                        createPathGoesData function is the same on the bucket.
                The variable path is create with the createPathGoesData
            The singleFile is the filename from the data in the bucket 'noaa-goes16'
            and the singleSize is the real size from the same singleFile on the bucket.
    And check if the size on the singleFile on the path created with the createPathGoesData is
                          equal to the original file on the bucket.

    """
    
    from goespy import os

    if checkFiles(path, singleFile):
        if os.path.getsize(path + singleFile) == singleSize:
            return True
        else:
            return False



""" That module contents the functions necessary to downloader the ABI-sensors and GLM-L2 Total Lightning
     from the GOES satellite """


def ABI_Downloader(bucket, year, month, day, hour, product, channel):
    """        ABI_Downloader(bucket,year,month,day,hour,product,channel): All these variables are strings.
    The first argument is the Bucket it's the reposity where has the contents from the satellite, example:  
    
    Bucket='noaa-goes16'
    year  = can be List or a single string to Year date: example = ['2017','2018'] or "2018
    month = can be List or a single string to month date: example = ['03','04'] or "03"
    day   = can be List or a single string for day date: example = ['10','20','30'] or "20"
    hour   = can be List or a single string to hour, and need be UTC coordinate time date: example = ['06','12','18'] or "06"
    product = can be a List or a single string for ABI sensors products from GOES satellite next-generation example: ["ABI-L2-CMIPF"] or "ABI-L1b-RadF"
    channel = can be a List or a single string for the channels from ABI sensors. Example = ['01','02'] or "13"
    """
    from goespy.utils import __isAList
    from goespy.utils import ProgressPercentage
    from goespy import checkData
    from goespy import boto3
    from goespy import botocore

    julianDay = ''
    days = 0
    s3 = boto3.resource(
        's3',
        config=botocore.client.Config(signature_version=botocore.UNSIGNED))
    goes16 = s3.Bucket(bucket)

    year, month, day, product, hour, julianDay = __isAList(
        year, month, day, product, hour, julianDay=julianDay)

    ## for loop to all variable year (it's a list var)
    for i in year:

        ## same think above
        for mth in month:

            ## I used that while loop, because I think it's a solution more "presentable"
            ## that while will travel on the days list, same to the julianDay
            while days <= len(day) - 1 and days <= len(julianDay) - 1:
                for prod in product:
                    print("Downloading... the product %s " % prod)
                    for nindex in hour:
                        print("Downloading dataset to... {0} UTC".format(nindex))
                        ## all these loops it's necessary to travel all the lenght of the variables
                        ## maybe that's not a elegant solution.
                        ## s3.objects.filters it's a function from boto3 to list all keys on the bucket
                        ## using a prefix
                        objs = goes16.objects.filter(
                            Delimiter='/',
                            Prefix="{0}/{1}/{2}/{3}/".format(
                                prod, i, julianDay[days], nindex))
                        for object in objs:
                            ## the keys it's a "path"+"filename" in the bucket, solution
                            ### we need only the filename, that's why used the rsplit function.
                            filename = object.key.rsplit('/', 1)[1]
                            for ch in channel:
                                ## when the filename has the same Channel from the user channel variable
                                ## call the function from download, but before it's done somes check files and directory
                                if filename.partition(ch)[1] == ch:

                               #     creating the new directory where we will put the dataset from the bucket
                                   
                                    path = checkData.createPathGoesData(bucket,
                                        i, mth, day[days], prod, nindex, ch)
                                    
                                    #checking if the file exist on the new directory and your size
                                    if checkData.checkFiles(path, filename):
                                        
                                        if checkData.checkSize(
                                                path, filename, object.size):
                                        
                                            pass
                                        
                                        else:

                                            # Downloading the file with the boto3
                                            goes16.download_file(
                                                object.key, path + filename,Callback=ProgressPercentage(filename,object.size))
                                            print('\n')
                                    else:
                                    
                                        # Downloading the file with the boto3
                                        goes16.download_file(
                                            object.key, path + filename,Callback=ProgressPercentage(filename,object.size))
                                        print('\n')
                days += 1

    return 0


def GLM_Downloader(bucket, year, month, day, hour):
    """        GLM_Downloader(bucket,year,month,day,hour): All these variables are strings.
    The first argument is the Bucket it's the reposity where has the contents from the satellite, example:  

    Bucket='noaa-goes16'
    year  = type List for Year date: example = ['2017','2018'] 
    month = type List for month date: example = ['03','04']
    day   = type List for day date: example = ['10','20','30']
    hour   = type List for hour need be UTC coordinate time date: example = ['06','12','18']"""

    from goespy.utils import __isAList
    from goespy.utils import ProgressPercentage
    from goespy import checkData
    from goespy import boto3
    from goespy import botocore
    julianDay = ''
    s3 = boto3.resource(
        's3',
        config=botocore.client.Config(signature_version=botocore.UNSIGNED))
    goes16 = s3.Bucket(bucket)
    product = 'GLM-L2-LCFA'
    days = 00
    year, month, day, product, hour, julianDay = __isAList(
        year, month, day, product, hour, julianDay=julianDay)

    for i in year:
        for mth in month:
            while days <= len(day) - 1 and days <= len(julianDay) - 1:
                for prod in product:
                    print("Downloading... the product %s " % prod)
                    for nindex in hour:
                        print("Downloading... the dataset from {0} UTC".format(
                            nindex))
                        objs = goes16.objects.filter(
                            Delimiter='/',
                            Prefix="{0}/{1}/{2}/{3}/".format(
                                prod, i, julianDay[days], nindex))

                        #print("{0}/{1}/{2}/{3}/".format(prod,i,julianDay[days],nindex))
                        for object in objs:

                            filename = object.key.rsplit('/', 1)[1]
                            ## creating the directory where will put the dataset from the bucket
                            pathFile = checkData.createPathGoesData(
                                bucket, i, mth, day[days], prod, nindex)

                            # checking if the data exist and your size!!!
                            if checkData.checkFiles(pathFile, filename):
                                if checkData.checkSize(pathFile, filename,
                                                       object.size):
                                    pass

                                else:
                                                                        
                                    # Downloading the file with the boto3
                                    goes16.download_file(object.key,pathFile+filename,Callback=ProgressPercentage(filename,object.size))
                                    print('\n')
                            else:
                                
                                # Downloading the file with the boto3
                                goes16.download_file(object.key,pathFile+filename,Callback=ProgressPercentage(filename,object.size))
                                print('\n')

                days += 1

    return 0


# -*- coding: utf-8 -*-
from setuptools import setup
setup(
    name='goespy',
    version='0.1.1',
    url='https://github.com/palexandremello/goes-py',
    license='BSD 2-Clause License',
    author='Paulo Alexandre Mello',
    author_email='palexandremello@gmail.com',
    keywords='weather goes satellite meteorology noaa',
    description=u'A Python package can be useful to download dataset from goes satellite on AWS',
    packages=['goespy'],
    install_requires=['boto3','botocore','pathlib'],
    zip_safe=False)




""" Utilites uses on the goespy 

"""

def daytoJulian(year,month,day):
    from goespy import datetime

    i,j,k = 0,0,0
    julianDay=[]
    while i <= len(year)-1:
        while j <= len(month)-1:
            while k <= len(day)-1:
                julian = str(datetime.datetime.strptime('{0}-{1}-{2}'.format(year[i],month[j], day[k]),
                '%Y-%m-%d').timetuple().tm_yday).zfill(3)
                julianDay.append(julian)
                
                k += 1
            j+= 1
        i+=1
    
    
    return julianDay

def __isAList(year, month, day, product, hour, channel=None, julianDay=None):
    """ All these variables are strings. This function will "convert" every single string on a List of string, 
    to be use in the Foor loops: If all the arguments are List of string, the function will not convert to List.
        The first argument is the Bucket it's the reposity where has the contents from the satellite, example:  
    
    Bucket='noaa-goes16'
    year  =  single string to Year date: example =  "2018" -> ['2018']
    month =  single string to month date: example = "03" -> ['03']
    day   =  single string for day date: example =  "20" -> ['20']
    hour   = single string to hour, and need be UTC coordinate time date: example =  "06" -> ['06']
    product= single string for ABI sensors products from GOES satellite next-generation example:  "ABI-L1b-RadF" -> ['ABI-L1b-RadF']
    channel = single string for the channels from ABI sensors. Example =  "13" -> ['13']
    """

    if not isinstance(hour, (list, )):
        hour = [hour]
    if not isinstance(year, (list, )):
        year = [year]

    if not isinstance(month, (list, )):
        month = [month]

    if not isinstance(day, (list, )):
        day = [day]

    if not isinstance(product, (list, )):
        product = [product]

    if julianDay  == None:
        return year, month, day, product, hour
    else:
        if not isinstance(julianDay, (list, )):
            julianDay = daytoJulian(year,month,day)

    return year, month, day, product, hour, julianDay

class ProgressPercentage(object):
    def __init__(self, filename,objectSize):
        from goespy import threading
        self._filename = filename
        self._size = objectSize
        self._seen_so_far = 0
        self._lock = threading.Lock()

    def __call__(self, bytes_amount):
        from goespy import sys
        # To simplify we'll assume this is hooked up
        # to a single filename.
        with self._lock:
            self._seen_so_far += bytes_amount
            percentage = round((self._seen_so_far / self._size) * 100,2)
            sys.stdout.write("\r%s  %s / %s  (%.2f%%)" % (self._filename, str(float(self._seen_so_far/1e6)),
                                             str(float(self._size/1e6)), percentage ) )
            sys.stdout.flush()


def bannerDisplay(version):
    text='goes-py'
    ch='=' 
    length=90
    spaced_text = ' %s ' % text
    banner = spaced_text.center(length, ch)
    print("")
    print(banner)
    print(" A Python package can be useful to download dataset from goes satellite on AWS  %sv" % version)
    print("==========================================================================================")